from whois import whois
from checkdmarc.spf import get_spf_record, SPFRecordNotFound
from checkdmarc.dmarc import get_dmarc_record, DMARCRecordNotFound
import time
import tldextract
from tqdm import tqdm
import progressbar


def progress_bar():
    widgets = ["TESTING A PROGRESS BAR", progressbar.Bar()]
    bar = progressbar.ProgressBar(widgets=widgets).start()
    return bar


class Lookup(object):
    def __init__(self, host):
        self.host = host
        self.whois = dict(whois(host))
        self.dmarc = None
        self.spf = None


def lookup(validated_hosts):
    bar = progress_bar()
    lookups = []
    for key, value in validated_hosts.items():
        if key == "domains":
            print("[Looking up domains...]")
            if any(value):
                for host in tqdm(value):
                    lookedup = Lookup(host)
                    try:
                        lookedup.dmarc = dict(get_dmarc_record(host))["record"]
                    except DMARCRecordNotFound as error:
                        print("DMARC exception. Should start using raise :)")
                        lookedup.dmarc = error.args[0]
                    try:
                        lookedup.spf = dict(get_spf_record(host))["record"]
                    except SPFRecordNotFound as error:
                        print(
                            "SPF exception. Should start using raise :)",
                            error.args,
                            type(error.args),
                            error.args[0],
                        )
                        # this 'works', as in, error.args returns some valuable detail (DNS timeout) and the host to look up.
                        lookedup.spf = error.args[0]
                    lookups.append(lookedup)
                    time.sleep(1)
                    # bar.update()
        elif key == "email":
            print("[Looking up email addresses...]")
            if any(value):
                for host in tqdm(value):
                    domain = str.split(host, "@")[-1]
                    lookedup = Lookup(domain)
                    try:
                        lookedup.dmarc = dict(get_dmarc_record(domain))["record"]
                    except DMARCRecordNotFound as error:
                        print("DMARC exception. Should start using raise :)")
                    try:
                        lookedup.spf = dict(get_spf_record(domain))["record"]
                    except SPFRecordNotFound as error:
                        print("SPF exception. Should start using raise :)",
                        error.args,
                        type(error.args),
                        error.args[0],
                        )
                    lookups.append(lookedup)
                    time.sleep(1)
        elif key == "URL":
            print("[Looking up URLs...]")
            if any(value):
                for host in tqdm(value):
                    domain = tldextract.extract(host).domain
                    lookedup = Lookup(domain)
                    lookedup.dmarc = dict(get_dmarc_record(domain))["record"]
                    lookedup.spf = dict(get_spf_record(domain))["record"]
                    lookups.append(lookedup)
                    # TODO add VirusTotal-lookup and return score?
                    time.sleep(1)
        elif key == "hostname":
            print("Looking up hostnames...]")
            if any(value):
                for host in tqdm(value):
                    pass
        elif key == "IP":
            print("Looking up IPs...]")
            if any(value):
                for host in tqdm(value):
                    lookedup = Lookup(host)
                    lookups.append(lookup)
                    time.sleep(1)

        else:
            print("No hosts to look up in ", key)
        bar.update()

    return lookups


def lookup_spf(validated_hosts):
    pass


def lookup_dmarc(validated_hosts):
    pass
